{ 4A20 DEMO SOFTWARE - DMA MODE 4A20 READ AND PRINT ROUTINE }
{$N-}    {No numeric coprocessor}
{$R+}    {Range checking on}
{$S+}    {Stack checking on}
{$A+}    {word alignment (Important! DMATarget needs to be on word boundary) }
{$DEFINE main}
uses CRT,DOS;

{$UNDEF DMADebug}        { it works ! (finally) }
{$I 4A20LOW }                 { include low level 4A20 I/O }
{ program 4A20DMA }

const
NumberOfChannels   = 6;                        { channels read before repeating }
NumberOfScans      = 14;                      { number of times we read the selected channels }
DMABufferLength    = NumberOfChannels * NumberOfScans;                      { 128 samples recorded }
PrintedDataSize    = 8;                        { character length of printed data }

type
DMABuffer = array [0..DMABufferLength] of word;
DMABufferPtr = ^DMABuffer;

var
DMAChannel         : byte;                        { DMA channel we use}
DMATargetPtr : DMABufferPtr;
DMATargetAddress : longint;
DMATargetSegment : longint;
DMATargetOffset : longint;
Scan : word;
Channel : word;
VectorSave : pointer;
ADCdata : longint;
GainSet : byte;
ConvertRate : longint;                    { the convert clock }
ConvertPeriod : word;

procedure AllocateDMATargetArray;              { allocate enough space for DMATarget }
var                                            { and make sure that it doesn't cross }
offset : word;                                 { a 64K page boundary (PC DMA can't)  }
segment : word;
bytesTillEOP : word;
dummySize : word;
dummyPtr : pointer;
begin
  segment := seg(HeapPtr^);
  offset  := ofs(HeapPtr^);
  bytesTillEOP := $FFFF-(((segment and $FFF) shl 4) + offset)+1;   { calculate bytes left till end of 64K page }
  if bytesTillEOP <= (DMABufferLength * 2) then        { if not enough space till end of 64K page }
  begin
    dummySize := bytesTillEOP;                 { allocate space till end of this 64K page }
    getmem(dummyPtr,dummySize);
    writeln('Not enough room on current 64K page for DMA buffer');
    writeln('had to allocate dummy space.');
    new(DMATargetPtr);                         { then allocate our DMA target buffer }
    freemem(dummyPtr,dummySize);               { finally, reclaim the space we wasted }
  end { if }
  else
  begin
    new(DMATargetPtr);                         { if there's enough room, just allocate it }
  end; { else }
end;


procedure PrintTheData;
var
index : word;
spaces : word;
begin
  gotoXY(1,1);
  writeln;
  write('channel' : PrintedDataSize);          { print channel numbers }
  for Channel := 0 to NumberOfChannels -1 do
  begin
    write(Channel : PrintedDataSize);
  end;
  writeln;
  writeln;
  for  Scan := 1 to NumberOfScans do
  begin
    write('scan' : PrintedDataSize div 2);
    write(Scan : PrintedDataSize div 2);
    for Channel := 0 to NumberOfChannels -1 do
    begin
      index := ((scan-1) * NumberOfChannels) + Channel;
      ADCData := DMATargetPtr^[index];
      if ADCData >32767 then ADCData := ((ADCdata and $7FFF) -32768);
      if (scan=1) and (channel=0) then
      write('XXXXX' : PrintedDataSize) { indicate bogus data }
      else write(ADCData : PrintedDataSize);
{      for spaces := 1 to (PrintedDataSize - 4) do write(' ');
      HexWordPrint(word(ADCData));}
    end; { for Channel }
    writeln;
  end; { for Scan }
end; { PrintTheData }

procedure CheckFor4A20;
begin
  if not CheckGCTable then
  begin
    writeln('Could not find 4A20... aborting program');
    halt(2);
  end; { if }
end; { Check for 4A20 }

procedure ReadPrint;
begin
  FillGCTableWithOffsetChannels(NumberOfChannels,GainSet);
  { we load table with 1 word offset to compensate for }
  { A-D pipelining }
  LoadScanLengthCounter(NumberOfChannels);     { should we want non-binary scan lengths }
  LoadRateCounter(ConvertPeriod);              { set the rate }
  SetMain4A20Mode(DMAMode0);
  WriteSetup(7,DMAChannel,0);
  SetDMAMode(DMAChannel,IOToMemory or AddrIncrement or SingleMode);
  DMATargetSegment := seg(DMATargetPtr^);
  DMATargetOffset := ofs(DMATargetPtr^);
  DMATargetAddress := DMATargetSegment shl 4 + DMATargetOffset;
  SetDMAAddress(DMAChannel,DMATargetAddress);
  SetDMALength(DMAChannel,DMABufferLength * 2);
  UnMaskDMAChannel(DMAChannel);                { and away we go }
  StartConvertClock;                           { start conversions }
  GotoXY(1,24);
  writeln('Acquiring Data');
  while not DMAIsDone(DMAChannel) do { nothing };      { wait till DMA is done }
  MaskDMAChannel(DMAChannel);                  { just in case... }
  StopConvertCLock;                            { stop the clock }
  SetMain4A20Mode(PolledMode);                  { disable IRQ and DRQ drivers }
  PrintTheData;                                { and display the results }
end;

procedure BumOut(s:string);
begin
  writeln(s);
  halt(2);
end;

procedure GetParms;
var
numok : integer;
rate : longint;
tempw : word;
begin
  ConvertRate := 50000;
  val(Paramstr(1),rate,numok);
  if (numok=0) and (rate >200) and (rate < 120000) then
  begin
    ConvertRate := rate;
    writeln('Conversion rate set to ',rate);
  end
  else writeln('Conversion rate default = ',ConvertRate);
  ConvertPeriod := RateClock div ConvertRate;
  DMAChannel := 3;
  if length(ParamStr(2)) > 0 then
  begin
    val(Paramstr(2),tempw,numok);
    if numok <> 0 then BumOut('Bad DMA channel');
    if tempw < 1 then Bumout('Bad DMA channel');
    if tempw > 7 then Bumout('Bad DMA channel');
    if tempw = 4 then Bumout('Bad DMA channel');
    DMAChannel := tempw;
  end;
  if DMAChannel > 3 then writeln('Using 16 bit DMA') else writeln('Using 8 bit DMA');
end;



begin
  DirectVideo := false;
  CheckFor4A20;
  GetParms;
  writeln;
  writeln('4A20 DMA driven demo');
  writeln('Convert rate is ',ConvertRate,' Hz using DMA channel ',DMAChannel);
  writeln('Scanning ',NumberOfChannels,' channels ',NumberOfScans,' times');
  writeln('for a total of ',NumberOfChannels * NumberOfScans,' conversions');
  MaskDMAChannel(DMAChannel);                  { just in case... }
  delay(1000);
  AllocateDMATargetArray;
  Reset4A20;                                   { do full reset }
  GainSet := 0;                                { for gain of 1 }
  ClrScr;
  while not keypressed do ReadPrint;
  GotoXY(1,24);
end.