{ 4A20 DEMO SOFTWARE - DMA MODE 4A20 READ AND PRINT ROUTINE }
{$N-}    {No numeric coprocessor}
{$R+}    {Range checking on}
{$S+}    {Stack checking on}
{$A+}    {word alignment (Important! DMATarget needs to be on word boundary) }
{$UNDEF DMADebug}        { it works ! (finally) }
{$I 4A20LOW }                 { include low level 4A20 I/O }

{ program 4A20DMA }

const
NumberOfChannels   = 6;                        { channels read before repeating }
NumberOfScans      = 14;                       { number of times we read the selected channels }
DMABufferLength    = NumberOfChannels * NumberOfScans;                      { 128 samples recorded }
ConvertRate        = 5000;                    { the convert clock }
ConvertPeriod      = RateClock div ConvertRate;
PrintedDataSize    = 8;                        { character length of printed data }
DMAChannel         = 1;                        { we'll use DMA channel 1 }

type
DMABuffer = array [0..DMABufferLength] of word;
DMABufferPtr = ^DMABuffer;

var
DMATargetPtr : DMABufferPtr;
DMATargetAddress : longint;
DMATargetSegment : longint;
DMATargetOffset : longint;
Scan : word;
Channel : word;
VectorSave : pointer;
ADCdata : longint;
GainSet : byte;
Bipolar : boolean;

procedure AllocateDMATargetArray;              { allocate enough space for DMATarget }
var                                            { and make sure that it doesn't cross }
offset : word;                                 { a 64K page boundary (PC DMA can't)  }
segment : word;
bytesTillEOP : word;
dummySize : word;
dummyPtr : pointer;
begin
  segment := seg(HeapPtr^);
  offset  := ofs(HeapPtr^);
  bytesTillEOP := $FFFF-(((segment and $FFF) shl 4) + offset)+1;   { calculate bytes left till end of 64K page }
  if bytesTillEOP <= (DMABufferLength * 2) then        { if not enough space till end of 64K page }
  begin
    dummySize := bytesTillEOP;                 { allocate space till end of this 64K page }
    getmem(dummyPtr,dummySize);
    writeln('Not enough room on current 64K page for DMA buffer');
    writeln('had to allocate dummy space.');
    new(DMATargetPtr);                         { then allocate our DMA target buffer }
    freemem(dummyPtr,dummySize);               { finally, reclaim the space we wasted }
  end { if }
  else
  begin
    new(DMATargetPtr);                         { if there's enough room, just allocate it }
  end; { else }
end;


procedure PrintTheData;
var index : word;
begin
  writeln;
  write('channel' : PrintedDataSize);          { print channel numbers }
  for Channel := 0 to NumberOfChannels -1 do
  begin
    write(Channel : PrintedDataSize);
  end;
  writeln;
  writeln;
  for  Scan := 1 to NumberOfScans do
  begin
    write('scan' : PrintedDataSize div 2);
    write(Scan : PrintedDataSize div 2);
    for Channel := 0 to NumberOfChannels -1 do
    begin
      { read the array from top to bottom because of DMA direction }
      index := DMABufferLength - ((scan-1) * NumberOfChannels + Channel) - 1;
      ADCData := DMATargetPtr^[index];
      if Bipolar = true then ADCData := ADCData -32768;
      write(ADCData : PrintedDataSize);
    end; { for Channel }
    writeln;
  end; { for Scan }
end; { PrintTheData }

procedure CheckFor4A20;
begin
  if not CheckGCTable then
  begin
    writeln('Could not find 4A20... aborting program');
    halt(2);
  end; { if }
end; { Check for 4A20 }

begin  { main }
  CheckFor4A20;
  writeln;
  writeln('4A20 DMA driven demo');
  writeln('Convert rate is ',ConvertRate,' Hz using DMA channel ',DMAChannel);
  writeln('Scanning ',NumberOfChannels,' channels ',NumberOfScans,' times');
  writeln('for a total of ',NumberOfChannels * NumberOfScans,' conversions');
  MaskDMAChannel(DMAChannel);                  { just in case... }
  AllocateDMATargetArray;
  Reset4A20;                                   { do full reset }
  GainSet := 0;                                { for gain of 1 }
  FillGCTableWithSequentialChannels(GainSet);  { for sequential channel sequence }
  LoadScanLengthCounter(NumberOfChannels);     { should we want non-binary scan lengths }
  LoadRateCounter(ConvertPeriod);              { set the rate }
  SetMain4A20Mode(DMAMode0);
  Bipolar := true;
  SetDMAMode(DMAChannel,IOToMemory or AddrDecrement or SingleMode or AutoInitMode);{ note that we decrement because }
  DMATargetSegment := seg(DMATargetPtr^);
  DMATargetOffset := ofs(DMATargetPtr^);
  DMATargetAddress := DMATargetSegment shl 4 + DMATargetOffset;
  { since were going backwards, we load DMA controller with top address (-1)  }
  DMATargetAddress := (DMATargetAddress + (DMABufferLength * 2)) - 1;
  SetDMAAddress(DMAChannel,DMATargetAddress);
  SetDMALength(DMAChannel,DMABufferLength * 2);
  SetIRQDRQ(0,DMAChannel);
  UnMaskDMAChannel(DMAChannel);                { and away we go }
  StartConvertClock;                           { start conversions }
  writeln('Acquiring Data Forever...');
  halt(1);
end.

