{ 4A20 DEMO SOFTWARE - INTERRUPT MODE 4A20 READ AND PRINT ROUTINE }
{$N-}    {No numeric coprocessor}

{$I 4A20LOW }                 { include low level 4A20 I/O }

{ program interrupt driven 4A20 }

const
{ the hardware interrupt we are using }
TheInterrupt       = 7;                        { demo program uses int 7 ( normally unused parallel port interrupt ) }
NumberOfScans      = 12;                       { number of times we read the selected channels }
NumberOfChannels   = 7;                        { channels read before repeating }
DataTableLength    = NumberOfChannels * NumberOfScans;                      { 128 samples recorded }
InterruptRate      = 500;                      { the interrupt rate ( has to be slow for pascal ISR ) }
InterruptPeriod    = RateClock div InterruptRate;
PrintedDataSize    = 8;                        { character length of printed data }

var
foo : string;
loops : longint;
isrcnt : word;
DataTable : array[0..DataTableLength] of word;
VectorSave : pointer;
DataTablePointer : word;
ADCdata : longint;
GainSet : byte;


procedure InterruptServiceRoutine;
interrupt;
begin
  DataTable[DataTablePointer] := IntTableConverterRead;
  DataTablePointer := DataTablePointer + 1;   { increment the acquired data pointer }
  if DataTablePointer = DataTableLength then
  begin
    StopConvertClock;                       { if at end of data block, stop clock }
  end;
  isrcnt := isrcnt +1;
  if (TheInterrupt <= 7) then IssueEOI else IssueEOI2;
end; { InterruptServiceRoutine }

procedure CheckFor4A20;
begin
  if not CheckGCTable then
  begin
    writeln('Could not find 4A20... aborting program');
    halt(2);
  end; { if }
end; { Check for 4A20 }

procedure PrintTheData;
var
channel : word;
scan : word;
begin
  writeln;
  write('Channel' : PrintedDataSize);          { print channel numbers }
  for channel := 0 to NumberOfChannels -1 do
  begin
    write(channel : PrintedDataSize);
  end;
  writeln;
  writeln;
  for  scan := 1 to NumberOfScans do
  begin
    write('scan' : PrintedDataSize div 2);
    write(scan : PrintedDataSize div 2);
    for channel := 0 to NumberOfChannels -1 do
    begin
      ADCData := DataTable[(scan-1) * NumberOfChannels + Channel];
      if ADCData >32767 then ADCData := ((ADCdata and $7FFF) -32768);
      if (scan=1) and (channel=0) then
      write('XXXXX' : PrintedDataSize) { indicate bogus data }
      else write(ADCData : PrintedDataSize);
    end; { for channel }
    writeln;
  end; { for scan }
end; { print the data }

begin  { main }
  CheckFor4A20;
  writeln;
  writeln('4A20 interrupt driven demo');
  writeln('Interrupt rate is ',InterruptRate,' Hz using interrupt ',TheInterrupt);
  writeln('Scanning ',NumberOfChannels,' channels ',NumberOfScans,' times');
  writeln('for a total of ',NumberOfChannels * NumberOfScans,' conversions');
  writeln;
  MaskInterrupt(TheInterrupt);
  StopConvertClock;                            { just in case }
  Reset4A20;                                   { do full reset }
  GetHardIntVec(TheInterrupt,VectorSave);
  SetHardIntVec(TheInterrupt,@InterruptServiceRoutine);
  GainSet := ADGain1;                                { for gain of 1 }
  FillGCTableWithOffsetChannels(NumberOfChannels,GainSet);      { for reading all 8 differential channels }
  LoadScanLengthCounter(NumberOfChannels);     { should we want non-binary lengths }
  LoadRateCounter(InterruptPeriod);            { set the rate }
  DataTablePointer := 0;                       { start at 0 in the table }
  SetMain4A20Mode(InterruptMode);
  Clr4A20IIRQ;                                 { make sure IRQ FF is cleared }
  WriteSetup(TheInterrupt,0,0);                { this also clears the table pointer }
  isrcnt := 0;
  UnMaskInterrupt(TheInterrupt);
  StartConvertClock;                           { start conversions }
  writeln('Acquiring Data');
  writeln;
  loops := 0;
  while ((port[ADStartReg] and ADStartMask) <> 0) and (loops < 200000) do loops := loops +1;
   { nothing };    { we are waiting for the acquisition }
                                               { phase to complete ( when interrupt service stops clock ) }
  MaskInterrupt(TheInterrupt);                 { now that were done, prevent anymore ints }
  SetMain4A20Mode(PolledMode);                  { disable IRQ driver }
  StopConvertClock;
  PrintTheData;                                { print out the data }
  if loops >= 200000 then writeln('ISR Count = ',isrcnt);
  halt(1);
end.
