{8237 DMA controller low level stuff}
{{$Define DMADEBUG}
type
LongIntRec =
record
  LoWord : word;
  HiWord : word;
end;

Const
{ define DMA page register location }

DMAPageRegBase  : byte = $80;
FirstDMABase : word = $0000;
SecondDMABase : word = $00C0;

{ things that apply to the DMACommandReg. Note that this is a global (all channel) register }

EnableDMA       : byte = $00;                  { to turn on DMA when done changing things }
DisableDMA      : byte = $04;                  { to turn off DMA while changing things }
CompTiming      : byte = $08;                  { compressed timing }
RotPriority     : byte = $10;                  { rotating priority }
ExtenWrite      : byte = $20;                  { extended write timing }
DRQLow          : byte = $40;                  { DRQ active low }
DackHigh        : byte = $80;                  { DACK active low }

{ things that apply to the DMAModeReg. Note that this is a per-channel register }

VerifyMode      : byte = $00;                  { verify transfer mode }
IOToMemory      : byte = $04;                  { I/O to memory tranfer mode }
MemoryToIO      : byte = $08;                  { memory to I/O tranfer mode }
AutoInitMode    : byte = $10;                  { Autoinitialize enable }
AddrDecrement   : byte = $20;                  { basakwards }
AddrIncrement   : byte = $00;                  { forwards }
DemandMode      : byte = $00;                  { demand transfer mode }
SingleMode      : byte = $40;                  { single transfer mode }
BlockMode       : byte = $80;                  { block transfer mode }

{ things that apply to the DMAMaskBitReg }

SetMaskBit      : byte = $04;                  { bit to set specified mask }
ClrMaskBit      : byte = $00;                  { to clear mask bit }

{ things that apply to the DMAStatusReg }

DMADoneMask    : byte =  $01;

var
DMABaseAddrReg   : word;
DMACountReg      : word;
DMACommandReg    : word;
DMAStatusReg     : word;
DMARequestReg    : word;
DMAMaskBitReg    : word;
DMAModeReg       : word;
DMAClearBPFF     : word;
DMAMasterReset   : word;
DMAClrAllMasks   : word;
DMASetAllMasks   : word;
DMAMaskReg       : word;

{************************** 8237 DMA RELATED ROUTINES **********************}

procedure SelectDMAChannel(channel : byte);
{ Next define the 8237 DMA controller registers }
var dmacontloc : word;
begin
  dmacontloc := FirstDMABase;
  if channel >3 then
  begin
    dmacontloc := SecondDMABase;
    DMABaseAddrReg  := dmacontloc + $00;
    DMACountReg     := dmacontloc + $02;
    DMACommandReg   := dmacontloc + $10;
    DMAStatusReg    := dmacontloc + $10;
    DMARequestReg   := dmacontloc + $12;
    DMAMaskBitReg   := dmacontloc + $14;
    DMAModeReg      := dmacontloc + $16;
    DMAClearBPFF    := dmacontloc + $18;
    DMAMasterReset  := dmacontloc + $1A;
    DMAClrAllMasks  := dmacontloc + $1C;
    DMASetAllMasks  := dmacontloc + $1E;
    DMAMaskReg      := dmacontloc + $1E;
  end
  else
  begin
    DMABaseAddrReg  := dmacontloc + $00;
    DMACountReg     := dmacontloc + $01;
    DMACommandReg   := dmacontloc + $08;
    DMAStatusReg    := dmacontloc + $08;
    DMARequestReg   := dmacontloc + $09;
    DMAMaskBitReg   := dmacontloc + $0A;
    DMAModeReg      := dmacontloc + $0B;
    DMAClearBPFF    := dmacontloc + $0C;
    DMAMasterReset  := dmacontloc + $0D;
    DMAClrAllMasks  := dmacontloc + $0E;
    DMASetAllMasks  := dmacontloc + $0F;
    DMAMaskReg      := dmacontloc + $0F;
  end;
end;

procedure StopDMA;                             { Global DMA stop to use when messing with modes }
begin
  Port[DMACommandReg] := DisableDMA;
end;

procedure StartDMA;                            { Global DMA start to use when done messing with modes }
begin
  Port[DMACommandReg] := EnableDMA;
end;

procedure SetDMAMode(channel : byte; mode : byte);
begin
  StopDMA;
  port[DMAModeReg] := ( mode or (channel and 3));       { set mode of selected channel }
  StartDMA;
end;

procedure MaskDMAChannel(channel : byte);
begin
  Port[DMAMaskBitReg] := SetMaskBit or (channel and 3);
end;

procedure UnMaskDMAChannel(channel : byte);
var mask : byte;
begin
  Port[DMAMaskBitReg] := ClrMaskBit or (channel and 3);
end;

function DMAIsDone(channel : byte): boolean;
begin
 DMAIsDone := (port[DMAStatusReg] and (DMADoneMask shl (channel and 3)) <> 0);
end;

procedure ClearBPFF;
begin
  port[DMAClearBPFF] := 0;
end;

procedure SetDMAPageAddress(channel : byte; dmapage : byte);
var
pagereg : byte;
begin
  case channel of
    0 : pagereg := $7 + DMAPageRegBase;
    1 : pagereg := $3 + DMAPageRegBase;
    2 : pagereg := $1 + DMAPageRegBase;
    3 : pagereg := $2 + DMAPageRegBase;
    5 : pagereg := $B + DMAPageRegBase;
    6 : pagereg := $9 + DMAPageRegBase;
    7 : pagereg := $A + DMAPageRegBase;
  else pagereg  := $2 + DMAPageRegBase;         { select the correct Page register }
  end; { case }                                { with channel 3 if bozo }
{$IFDEF DMADebug}
    writeln('');
    writeln('>>>>> SetDMAPageAddress <<<<<<');
    writeln('writing ',dmapage,' to page register at ',pagereg);
{$ENDIF}
  port[pagereg] := dmapage;                    { set the page register }
end; { SetDMAPageAddress }


procedure SetDMAAddress(channel : byte; address : longint);
var
lowByte : byte;
highByte : byte;
topByte : byte;
templong : longint;
chansel : byte;
begin
  if channel > 3 then
  begin  {16 bit DMA case}
    chansel := ((channel and 3) shl 2); { DMA channel regs port calculation }
    address := address shr 1; {make word address out of byte address}
    topByte := (address shr 15) and $00FF;
  end
  else
  begin  {8 bit DMA case}
    chansel := ((channel and 3) shl 1);
    topbyte :=lo(LongIntRec(address).HiWord);   {shr by 16 dont work on 6117!}
  end;
  lowByte := byte(address and $00FF);
  highByte := byte((address shr 8) and $00FF);
{$IFDEF DMADebug}
    writeln('');
    writeln('>>>>> SetDMAAddress <<<<<<');
    writeln('longint DMA address = ',address);
    writeln('low byte of DMA address = ',lowByte);
    writeln('high byte of DMA address = ',highByte);
    writeln('top byte of DMA address = ',topbyte);
{$ENDIF}
  asm
    cli
  end;                                         { disable interrupts so that BPFF is not touched }
  ClearBPFF;                                   { start with byte pointer flip flop pointing at LSB }
  port[DMABaseAddrReg or chansel] := lowByte;  { write low byte }
  port[DMABaseAddrReg or chansel] := highByte; { write high byte }
  asm
    sti
  end;
  SetDMAPageAddress(channel,TopByte);        { set the page }
end; { SetDMAAddress }

procedure SetDMALength(channel : byte; length : word);
var
lowByte : byte;
highByte : byte;
chansel : byte;
begin
  if channel > 3 then
  begin  {16 bit DMA case}
    length := length div 2;   { transfer lengths are in bytes, so div by two}
    chansel := ((channel and 3) shl 2); { DMA channel regs port calculation }
  end
  else
  begin  {8 bit DMA case}
    chansel := ((channel and 3) shl 1);
  end;                                              { on 16 bit channels }
  length := length -1;                          { DMA controller transfers 1 more than the programmed count }
  lowByte := lo(length);                        { note that to transfer 65536 bytes, length = 0 !! }
  highByte := hi(length);
{$IFDEF DMADebug}
    writeln;
    writeln('>>>>> SetDMALength <<<<<<');
    writeln('low byte of DMA length = ',lowByte);
    writeln('high byte of DMA length = ',highByte);
{$ENDIF}
  asm
    cli
  end;                                         { disable interrupts so that BPFF is not touched }
  ClearBPFF;                                   { start with byte pointer flip flop pointing at LSB }
  port[DMACountReg or chansel] := lowByte;     { write low byte }
  port[DMACountReg or chansel] := highByte;    { write high byte }
  asm
    sti
  end;
end; { SetDMALength }

