
{ 8254 low level procedures }
const

{$ifndef CounterBase}
Counterbase = $300;
{$endif}

{ 8254 counters }
CounterA    = CounterBase + $0;
CounterB    = CounterBase + $1;
CounterC    = CounterBase + $2;
CounterCont = CounterBase + $3;         { Normally not used - generated }

{ Then define the counter mode control bytes }

PeriodicInt    : byte = $0;
HardwOneShot   : byte = $1;
RateGenerator  : byte = $2;
SquareWave     : byte = $3;
SoftwStrobe    : byte = $4;
HardwStrobe    : byte = $5;

{ Some useful counter masks and command bytes }

ReadBackCommand  : byte = $EE;
OutBitMask     : byte = $80;
NullCountMask  : byte = $40;

procedure SetCounterMode(counterPort : word; counterMode : byte);
var modeByte : byte;
var controlPort : word;

begin
  modeByte := lo(counterPort shl 6);          { generate counter select bits }
  modeByte := modeByte or counterMode shl 1;  { or in mode bits }
  modeByte := modeByte or $30;                { set RW bits for least - most, binary }
  controlPort := counterPort or $3;           { generate control port address }
  port[controlPort] := modeByte ;             { write mode byte }
end; { SetCounterMode }

procedure LoadCounter(counterPort : word; count : word);
var loByte : byte ;
var hiByte : byte ;

begin
  loByte := lo(count);                        { Least significant byte }
  hibyte := hi(count);                        { Most significant byte }
  port[counterPort] := loByte;                { Write least }
  port[counterPort] := hiByte;                { Write most }
end; { LoadCounter }

procedure ReadCounter(counterPort : word; var count : word);
var loByte : word ;
var hiByte : word ;

begin
  loByte := port[counterPort];                { Read least significant byte }
  hiByte := port[counterPort];                { Read most significant byte }
  count  := lobyte and $00FF;                 { Build word }
  count  := swap(count);
  count  := count or (hiByte and $00FF);
  count  := swap(count);
end; { LoadCounter }

procedure LatchCounter(counterPort : word);
var modebyte : byte;
var controlPort : word ;

begin
  modeByte := lo(counterPort shl 6);          { generate counter select bits with RW bits =0 }
  controlPort := counterPort or $3;           { generate control port address }
  port[controlPort] := modeByte ;             { write latch mode word }
end; { LatchCounter }

procedure ReadCounterStatus(counterPort : word; var status : byte);
var controlPort : word ;
var counterSelect : byte ;
var counterSelectBits : byte ;
begin
  controlPort := counterPort or $3;           { Generate control port address }
  counterSelect := (counterPort and 3);
  counterSelectBits := 2;
  counterSelectBits := counterSelectBits shl counterSelect;
  port[controlPort] := ReadBackCommand or counterSelectBits;       { latch status command }
  status := port[counterPort];                { Read status byte }
end; { ReadCounterStatus }

